@@ -1,7 +1,7 @@
-use core::ffi::{CStr, c_int, c_long, c_void};
+use core::ffi::{c_int, c_long, c_void};
 use core::ptr::{NonNull, null_mut as NULL};
 use pyo3_ffi::*;
-use std::fmt::{self, Display, Formatter};
+use std::fmt::{Display, Formatter};
 
 use crate::{
     classes::{
@@ -12,7 +12,12 @@ use crate::{
         time::Time,
         time_delta::TimeDelta,
     },
-    common::{parse::Scan, rfc2822, round, scalar::*},
+    common::{
+        fmt::{self, Suffix},
+        parse::Scan,
+        rfc2822, round,
+        scalar::*,
+    },
     docstrings as doc,
     py::*,
     pymodule::State,
@@ -28,8 +33,6 @@ pub(crate) struct OffsetDateTime {
     pub(crate) offset: Offset,
 }
 
-pub(crate) const SINGLETONS: &[(&CStr, OffsetDateTime); 0] = &[];
-
 impl OffsetDateTime {
     pub(crate) const fn new_unchecked(date: Date, time: Time, offset: Offset) -> Self {
         OffsetDateTime { date, time, offset }
@@ -128,15 +131,6 @@ impl OffsetDateTime {
         OffsetDateTime::new(date, time, Offset::from_py(dt)?)
             .ok_or_value_err("Datetime is out of range")
     }
-
-    pub(crate) fn from_py_with_subsec(dt: PyDateTime, subsec: SubSecNanos) -> PyResult<Self> {
-        OffsetDateTime::new(
-            Date::from_py(dt.date()),
-            Time::from_py_dt_with_subsec(dt, subsec),
-            Offset::from_py(dt)?,
-        )
-        .ok_or_value_err("Datetime is out of range")
-    }
 }
 
 impl DateTime {
@@ -256,16 +250,19 @@ fn skip_tzname(s: &mut Scan) -> Option<()> {
     Some(())
 }
 
-impl PyWrapped for OffsetDateTime {}
+impl PySimpleAlloc for OffsetDateTime {}
 
 impl Display for OffsetDateTime {
-    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         let &OffsetDateTime { date, time, offset } = self;
         write!(f, "{date}T{time}{offset}")
     }
 }
 
 fn __new__(cls: HeapType<OffsetDateTime>, args: PyTuple, kwargs: Option<PyDict>) -> PyReturn {
+    if args.len() == 1 && kwargs.map_or(0, |d| d.len()) == 0 {
+        return parse_iso(cls, args.iter().next().unwrap());
+    }
     let mut year: c_long = 0;
     let mut month: c_long = 0;
     let mut day: c_long = 0;
@@ -301,13 +298,24 @@ fn __new__(cls: HeapType<OffsetDateTime>, args: PyTuple, kwargs: Option<PyDict>)
         .to_obj(cls)
 }
 
-fn __repr__(_: PyType, slf: OffsetDateTime) -> PyReturn {
-    let OffsetDateTime { date, time, offset } = slf;
-    format!("OffsetDateTime({date} {time}{offset})").to_py()
+fn __repr__(_: PyType, OffsetDateTime { date, time, offset }: OffsetDateTime) -> PyReturn {
+    PyAsciiStrBuilder::format((
+        b"OffsetDateTime(\"",
+        date.format_iso(false),
+        b' ',
+        time.format_iso(fmt::Unit::Auto, false),
+        offset.format_iso(false),
+        b"\")",
+    ))
 }
 
-fn __str__(_: PyType, slf: OffsetDateTime) -> PyReturn {
-    format!("{slf}").to_py()
+fn __str__(_: PyType, OffsetDateTime { date, time, offset }: OffsetDateTime) -> PyReturn {
+    PyAsciiStrBuilder::format((
+        date.format_iso(false),
+        b'T',
+        time.format_iso(fmt::Unit::Auto, false),
+        offset.format_iso(false),
+    ))
 }
 
 fn __richcmp__(
@@ -323,7 +331,6 @@ fn __richcmp__(
         let &State {
             instant_type,
             zoned_datetime_type,
-            system_datetime_type,
             ..
         } = cls.state();
 
@@ -331,8 +338,6 @@ fn __richcmp__(
             inst
         } else if let Some(zdt) = b_obj.extract(zoned_datetime_type) {
             zdt.instant()
-        } else if let Some(odt) = b_obj.extract(system_datetime_type) {
-            odt.instant()
         } else {
             return not_implemented();
         }
@@ -380,8 +385,6 @@ fn __sub__(obj_a: PyObj, obj_b: PyObj) -> PyReturn {
             inst
         } else if let Some(zdt) = obj_b.extract(state.zoned_datetime_type) {
             zdt.instant()
-        } else if let Some(odt) = obj_b.extract(state.system_datetime_type) {
-            odt.instant()
         } else if type_b == state.time_delta_type.into()
             || type_b == state.date_delta_type.into()
             || type_b == state.datetime_delta_type.into()
@@ -447,18 +450,6 @@ pub(crate) fn to_instant(cls: HeapType<OffsetDateTime>, slf: OffsetDateTime) ->
     slf.instant().to_obj(cls.state().instant_type)
 }
 
-pub(crate) fn instant(cls: HeapType<OffsetDateTime>, slf: OffsetDateTime) -> PyReturn {
-    // SAFETY: calling CPython API with valid arguments
-    unsafe {
-        PyErr_WarnEx(
-            PyExc_DeprecationWarning,
-            c"instant() method is deprecated. Use to_instant() instead".as_ptr(),
-            1,
-        )
-    };
-    to_instant(cls, slf)
-}
-
 fn to_fixed_offset(cls: HeapType<OffsetDateTime>, slf_obj: PyObj, args: &[PyObj]) -> PyReturn {
     match *args {
         [] => Ok(slf_obj.newref()),
@@ -478,26 +469,21 @@ fn to_fixed_offset(cls: HeapType<OffsetDateTime>, slf_obj: PyObj, args: &[PyObj]
 fn to_tz(cls: HeapType<OffsetDateTime>, slf: OffsetDateTime, tz_obj: PyObj) -> PyReturn {
     let &State {
         zoned_datetime_type,
-        exc_tz_notfound,
         ref tz_store,
         ..
     } = cls.state();
-    let tz = tz_store.obj_get(tz_obj, exc_tz_notfound)?;
-    slf.instant()
-        .to_tz(tz)
-        .ok_or_value_err("Resulting datetime is out of range")?
-        .to_obj(zoned_datetime_type)
+    let tz = tz_store.obj_get(tz_obj)?;
+    slf.instant().to_tz(tz, zoned_datetime_type)
 }
 
 fn to_system_tz(cls: HeapType<OffsetDateTime>, slf: OffsetDateTime) -> PyReturn {
     let &State {
-        py_api,
-        system_datetime_type,
+        zoned_datetime_type,
+        ref tz_store,
         ..
     } = cls.state();
-    slf.instant()
-        .to_system_tz(py_api)?
-        .to_obj(system_datetime_type)
+    let tz = tz_store.get_system_tz()?;
+    slf.instant().to_tz(tz, zoned_datetime_type)
 }
 
 pub(crate) fn unpickle(state: &State, arg: PyObj) -> PyReturn {
@@ -598,8 +584,47 @@ fn replace_time(
     }
 }
 
-fn format_common_iso(cls: PyType, slf: OffsetDateTime) -> PyReturn {
-    __str__(cls, slf)
+fn format_iso(
+    cls: HeapType<OffsetDateTime>,
+    slf: OffsetDateTime,
+    args: &[PyObj],
+    kwargs: &mut IterKwargs,
+) -> PyReturn {
+    fmt::format_iso(
+        slf.date,
+        slf.time,
+        cls.state(),
+        args,
+        kwargs,
+        Suffix::Offset(slf.offset),
+    )
+}
+
+fn parse_iso(cls: HeapType<OffsetDateTime>, arg: PyObj) -> PyReturn {
+    OffsetDateTime::parse(
+        arg.cast::<PyStr>()
+            // NOTE: this exception message also needs to make sense when
+            // called through the constructor
+            .ok_or_type_err("When parsing from ISO format, the argument must be str")?
+            .as_utf8()?,
+    )
+    .ok_or_else_value_err(|| format!("Invalid format: {arg}"))?
+    .to_obj(cls)
+}
+
+fn format_common_iso(
+    cls: HeapType<OffsetDateTime>,
+    slf: OffsetDateTime,
+    args: &[PyObj],
+    kwargs: &mut IterKwargs,
+) -> PyReturn {
+    deprecation_warn(c"format_common_iso() has been renamed to format_iso()")?;
+    format_iso(cls, slf, args, kwargs)
+}
+
+fn parse_common_iso(cls: HeapType<OffsetDateTime>, arg: PyObj) -> PyReturn {
+    deprecation_warn(c"parse_common_iso() has been renamed to parse_iso()")?;
+    parse_iso(cls, arg)
 }
 
 fn replace(
@@ -709,18 +734,6 @@ pub(crate) fn to_plain(cls: HeapType<OffsetDateTime>, slf: OffsetDateTime) -> Py
     slf.without_offset().to_obj(cls.state().plain_datetime_type)
 }
 
-pub(crate) fn local(cls: HeapType<OffsetDateTime>, slf: OffsetDateTime) -> PyReturn {
-    // SAFETY: calling CPython API with valid arguments
-    unsafe {
-        PyErr_WarnEx(
-            PyExc_DeprecationWarning,
-            c"local() method is deprecated. Use to_plain() instead".as_ptr(),
-            1,
-        )
-    };
-    to_plain(cls, slf)
-}
-
 pub(crate) fn timestamp(_: PyType, slf: OffsetDateTime) -> PyReturn {
     slf.instant().epoch.get().to_py()
 }
@@ -848,12 +861,10 @@ fn difference(cls: HeapType<OffsetDateTime>, slf: OffsetDateTime, arg: PyObj) ->
         inst
     } else if let Some(zdt) = arg.extract(state.zoned_datetime_type) {
         zdt.instant()
-    } else if let Some(odt) = arg.extract(state.system_datetime_type) {
-        odt.instant()
     } else {
         raise_type_err(
             "difference() argument must be an OffsetDateTime, 
-                Instant, ZonedDateTime, or SystemDateTime",
+                Instant, or ZonedDateTime",
         )?
     };
 
@@ -991,16 +1002,6 @@ fn from_timestamp_nanos(
     .to_obj(cls)
 }
 
-fn parse_common_iso(cls: HeapType<OffsetDateTime>, arg: PyObj) -> PyReturn {
-    OffsetDateTime::parse(
-        arg.cast::<PyStr>()
-            .ok_or_type_err("Expected a string")?
-            .as_utf8()?,
-    )
-    .ok_or_else_value_err(|| format!("Invalid format: {arg}"))?
-    .to_obj(cls)
-}
-
 fn parse_strptime(
     cls: HeapType<OffsetDateTime>,
     args: &[PyObj],
@@ -1029,7 +1030,7 @@ fn parse_strptime(
 }
 
 fn format_rfc2822(_: PyType, slf: OffsetDateTime) -> PyReturn {
-    let fmt = rfc2822::write(slf);
+    let fmt = rfc2822::format(slf);
     // SAFETY: we know the format is ASCII only
     unsafe { std::str::from_utf8_unchecked(&fmt[..]) }.to_py()
 }
@@ -1090,9 +1091,7 @@ static mut METHODS: &[PyMethodDef] = &[
         to_instant,
         doc::EXACTANDLOCALTIME_TO_INSTANT
     ),
-    method0!(OffsetDateTime, instant, c""), // deprecated alias
     method0!(OffsetDateTime, to_plain, doc::EXACTANDLOCALTIME_TO_PLAIN),
-    method0!(OffsetDateTime, local, c""), // deprecated alias
     method1!(OffsetDateTime, to_tz, doc::EXACTTIME_TO_TZ),
     method_vararg!(
         OffsetDateTime,
@@ -1112,16 +1111,10 @@ static mut METHODS: &[PyMethodDef] = &[
         parse_rfc2822,
         doc::OFFSETDATETIME_PARSE_RFC2822
     ),
-    method0!(
-        OffsetDateTime,
-        format_common_iso,
-        doc::OFFSETDATETIME_FORMAT_COMMON_ISO
-    ),
-    classmethod1!(
-        OffsetDateTime,
-        parse_common_iso,
-        doc::OFFSETDATETIME_PARSE_COMMON_ISO
-    ),
+    method_kwargs!(OffsetDateTime, format_iso, doc::OFFSETDATETIME_FORMAT_ISO),
+    method_kwargs!(OffsetDateTime, format_common_iso, c""), // deprecated alias
+    classmethod1!(OffsetDateTime, parse_iso, doc::OFFSETDATETIME_PARSE_ISO),
+    classmethod1!(OffsetDateTime, parse_common_iso, c""), // deprecated alias
     method0!(OffsetDateTime, timestamp, doc::EXACTTIME_TIMESTAMP),
     method0!(
         OffsetDateTime,
